@@ -83,6 +83,7 @@ swallow_t* new_swallow(void)
 	(*swallow).name = NULL;
 	(*swallow).window_name = NULL;
 	(*swallow).window = None;
+	(*swallow).pid = 0;
 	first_swallow = swallow;
 	return swallow;
 }
@@ -388,7 +389,7 @@ int parse_state(FILE* fp, state_t* state)
 
 void parse_config(const char* filename)
 {
-	FILE* fp = fopen("asexecrc", "r");
+	FILE* fp = fopen(filename, "r");
 	char* buf;
 	if (fp == NULL)
 	{
@@ -481,7 +482,7 @@ char* find_rcfile(void)
 		home = "";
 	rcfile = malloc(strlen(home) + strlen(afterdir) + strlen(NAME) + 4 + 1);
 	if (*afterdir == '~')
-		sprintf(rcfile, "%s/%s/%s", home, NAME, afterdir + 1);
+		sprintf(rcfile, "%s%s/%s", home, afterdir + 1, NAME);
 	else
 		sprintf(rcfile, "%s/%s", afterdir, NAME);
 	if ((fp = fopen(rcfile, "r")) != NULL)
