@@ -3,6 +3,7 @@
  */
 
 #include <ctype.h>
+#include <signal.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -125,14 +126,6 @@ void change_state(state_t* state)
 	{
 		/* unshape the window */
 		XShapeCombineMask(dpy, win, ShapeBounding, 0, 0, None, ShapeSet);
-#if 0
-		XRectangle rect;
-		rect.x = (Width - (*state).icon.width) / 2;
-		rect.y = (Height - (*state).icon.height) / 2;
-		rect.width = Width;
-		rect.height = Height;
-		XShapeCombineRectangles(dpy, win, ShapeBounding, 0, 0, &rect, 1, ShapeSet, Unsorted);
-#endif
 		is_shaped = 0;
 	}
 	/* change state */
@@ -202,9 +195,24 @@ void asexec_init2(int argc, char** argv)
 	XMapRaised(dpy, win);
 	/* start our children */
 	for (swallow = first_swallow ; swallow != NULL ; swallow = (*swallow).next)
-		if ((*swallow).exec != NULL && !fork())
-			if (execl ("/bin/sh", "/bin/sh", "-c", (*swallow).exec, NULL) == -1)
-				exit (100);
+		if ((*swallow).exec != NULL)
+		{
+			pid_t pid;
+			pid = fork();
+			if (pid < 0)
+				fprintf(stderr, "%s: unable to fork child process\n", MyName);
+			else if (pid == 0)
+			{
+				/* child process */
+				if (execl ("/bin/sh", "/bin/sh", "-c", (*swallow).exec, NULL) == -1)
+				{
+					fprintf(stderr, "%s: unable to start child '%s'\n", MyName, (*swallow).exec);
+					exit (100);
+				}
+			}
+			else
+				(*swallow).pid = pid;
+		}
 }
 
 Window find_client_window_recurse(Window w)
@@ -276,6 +284,9 @@ void send_clientmessage(Window window, Atom atom, Time timestamp)
 void asexec_done(void)
 {
 	swallow_t* swallow;
+	pid_t pid, p;
+	/* clean up child windows */
+	XUnmapWindow(dpy, win);
 	for (swallow = first_swallow ; swallow != NULL ; swallow = (*swallow).next)
 		if ((*swallow).window != None)
 		{
@@ -283,6 +294,18 @@ void asexec_done(void)
 			(*swallow).window = None;
 			XSync(dpy, 0);
 		}
+	XCloseDisplay(dpy);
+	/* spawn a child process to kill the other children */
+	pid = getpid();
+	p = fork();
+	if (p <= 0)
+	{
+		sleep(1);
+		/* we gave 'em a chance to die on their own - now kill 'em with prejudice */
+		for (swallow = first_swallow ; swallow != NULL ; swallow = (*swallow).next)
+			if ((*swallow).pid && getpgid((*swallow).pid) == pid)
+				kill((*swallow).pid, SIGKILL);
+	}
 	exit(0);
 }
 
